﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MathModTasks
{
    public struct Item : IComparable<Item>
    {
        public int number, aTime, bTime;

        public override string ToString()
        {
            return aTime + " " + bTime;
        }

        public int CompareTo(Item item)
        {
            if (aTime <= bTime)
            {
                if (aTime > item.aTime)
                    return 1;
                if (aTime < item.aTime)
                    return -1;
            }
            else
            {
                if (bTime > item.bTime)
                    return 1;
                if (bTime < item.bTime)
                    return -1;
            }
            return 0;
        }
    }

    public struct Activity
    {
        public int eventStart, eventEnd, time;
    }
    public struct Path
    {
        public string path;
        public int lastPoint, length;
    }

    public static class ReadSaveData
    {
        //Считывание данных для поиска критического и минимального пути
        //В первую строку записывается начало работы
        //Во вторую строку конец работы
        //В третью — ее длительность
        //В файле: начало | конец | длительность
        public static void ReadData(string path, ref List<Activity> activities)
        {
            if (!File.Exists(path))
            {
                Console.WriteLine("Файл не найден!");
                Console.ReadKey();
                Environment.Exit(0);
            }
            var lines = File.ReadAllLines(path);
            foreach (var line in lines)
            {
                string[] str = line.Split(';');
                activities.Add(new Activity { eventStart = Convert.ToInt32(str[0]), eventEnd = Convert.ToInt32(str[1]), time = Convert.ToInt32(str[2]) });
            }
        }

        /// <summary>
        /// Чтение данных из CSV-файла по указанному пути
        /// </summary>
        /// <param name="path">Путь к файлу</param>
        /// <returns></returns>
        /// <remarks>Транспортная задача: для чтения надо первую ячейку сделать НУЛЕМ! Остальные - как в таблице. Распределение работ между станками: данные в файле должны выглядеть как таблица, у которой значения ячеек разделены с помощью ";", без пробелов.</remarks>        
        static public List<string[]> ReadData(string path)
        {
            List<string[]> data = new List<string[]>();
            try
            {
                using (StreamReader sr = new StreamReader(path, Encoding.UTF8, true))
                {
                    while (sr.EndOfStream != true)
                    {
                        string[] str = sr.ReadLine().Split(';');

                        data.Add(str);
                    }
                }
                return data;
            }
            catch (FileNotFoundException)
            {
                Console.WriteLine("Ошибка считывания данных!");
                return data;
            }
        }


        //Метод для путей
        public static void WriteToFile(string path, Path savingPath)
        {
            if (!File.Exists(path)) File.Create(path).Close();
            try
            {
                using (StreamWriter sw = new StreamWriter(path, false))
                {
                    sw.WriteLine("Найденный путь имеет вид:");
                    sw.WriteLine(savingPath.path);
                    sw.WriteLine("Его длина составляет: " + savingPath.length);
                }
            }
            catch
            {
                Console.WriteLine("Не удалось записать данные в файл!");
                Console.ReadKey();
                Environment.Exit(0);
            }
        }
    }
}
